package ifc;

import addr.addrMac;
import pack.packHolder;
import ip.ipCor4;
import ip.ipCor6;
import ip.ipIfc4;
import ip.ipIfc6;
import util.counter;
import util.state;

/**
 * ethernet encapsulater
 *
 * @author matecsaba
 */
public class ifcEther {

    private final ifcEtherWorker se;

    private final ifcEtherWorker st;

    /**
     * parse ethernet header
     *
     * @param pck packet to parse
     * @param typ set true to process ethertype
     */
    public static void parseETHheader(packHolder pck, boolean typ) {
        pck.getAddr(pck.ETHtrg, 0);
        pck.getAddr(pck.ETHsrc, addrMac.size);
        pck.getSkip(addrMac.sizeX2);
        if (!typ) {
            return;
        }
        pck.ETHtype = pck.msbGetW(0);
        pck.getSkip(2);
    }

    /**
     * create ethernet header
     *
     * @param pck packet to parse
     * @param typ set true to process ethertype
     */
    public static void createETHheader(packHolder pck, boolean typ) {
        pck.putAddr(0, pck.ETHtrg);
        pck.putAddr(addrMac.size, pck.ETHsrc);
        pck.putSkip(addrMac.sizeX2);
        if (typ) {
            pck.msbPutW(0, pck.ETHtype);
            pck.putSkip(2);
        }
        pck.merge2beg();
    }

    /**
     * update ethernet header
     *
     * @param pck packet to update
     */
    public static void updateETHheader(packHolder pck) {
        int size = addrMac.sizeX2;
        if (pck.ETHtype >= 0) {
            size += 2;
        }
        pck.unMergeBytes(size);
        pck.putSkip(-size);
        pck.putAddr(0, pck.ETHtrg);
        pck.putAddr(addrMac.size, pck.ETHsrc);
        pck.putSkip(addrMac.sizeX2);
        if (pck.ETHtype >= 0) {
            pck.msbPutW(0, pck.ETHtype);
            pck.putSkip(2);
        }
        pck.putSkip(size);
        pck.mergeHeader(-1, pck.headSize() - size);
    }

    /**
     * guess ethertype of ip packet
     *
     * @param pck packet to inspect
     * @return ethertype, -1 if not valid
     */
    public static int guessEtherType(packHolder pck) {
        switch (pck.getByte(0) >>> 4) {
            case ipCor4.protocolVersion:
                return ipIfc4.type;
            case ipCor6.protocolVersion:
                return ipIfc6.type;
            default:
                return -1;
        }
    }

    /**
     * create new encapsulater
     *
     * @param notEther not ethernet encapsulation
     */
    public ifcEther(boolean notEther) {
        se = new ifcEtherWorker();
        st = new ifcEtherWorker();
        se.other = st;
        st.other = se;
        se.notEther = notEther;
        st.notEther = notEther;
        se.encDec = false;
        st.encDec = true;
    }

    /**
     * get encapsulated side
     *
     * @return interface handler
     */
    public ifcUp getSideEth() {
        return se;
    }

    /**
     * get ethertype side
     *
     * @return interface handler
     */
    public ifcUp getSideTyp() {
        return st;
    }

    /**
     * set filter
     *
     * @param prom pormiscous
     */
    public void setFilter(boolean prom) {
        se.lower.setFilter(prom);
        st.lower.setFilter(prom);
    }

}

class ifcEtherWorker implements ifcUp {

    public boolean encDec; // true=decapsulated, false=encapsulated

    public boolean notEther;

    public ifcEtherWorker other;

    public counter cntr = new counter();

    public ifcDn lower = new ifcNull();

    public void setParent(ifcDn parent) {
        lower = parent;
    }

    public void setState(state.states stat) {
    }

    public void closeUp() {
    }

    public counter getCounter() {
        return cntr;
    }

    public void recvPack(packHolder pck) {
        cntr.rx(pck);
        pck = pck.copyBytes(true, true);
        if (encDec) {
            if (notEther) {
                pck.ETHsrc = addrMac.getBroadcast();
                pck.ETHtrg = addrMac.getBroadcast();
            }
            ifcEther.createETHheader(pck, false);
        } else {
            ifcEther.parseETHheader(pck, false);
        }
        other.lower.sendPack(pck);
    }

}
